@@ -23,12 +23,12 @@ import xml.etree.ElementTree
 import textwrap
 from io import StringIO
 from collections import namedtuple
+from test.support import os_helper
 from test.support.script_helper import assert_python_ok
-from test.support import (
-    TESTFN, rmtree,
-    reap_children, reap_threads, captured_output, captured_stdout,
-    captured_stderr, unlink, requires_docstrings
-)
+from test.support import threading_helper
+from test.support import (reap_children, captured_output, captured_stdout,
+                          captured_stderr, requires_docstrings)
+from test.support.os_helper import (TESTFN, rmtree, unlink)
 from test import pydoc_mod
 
 
@@ -727,7 +727,7 @@ class PydocDocTest(unittest.TestCase):
         self.assertEqual(synopsis, expected)
 
     def test_synopsis_sourceless_empty_doc(self):
-        with test.support.temp_cwd() as test_dir:
+        with os_helper.temp_cwd() as test_dir:
             init_path = os.path.join(test_dir, 'foomod42.py')
             cached_path = importlib.util.cache_from_source(init_path)
             with open(init_path, 'w') as fobj:
@@ -744,11 +744,11 @@ class PydocDocTest(unittest.TestCase):
                          ('I Am A Doc', '\nHere is my description'))
 
     def test_is_package_when_not_package(self):
-        with test.support.temp_cwd() as test_dir:
+        with os_helper.temp_cwd() as test_dir:
             self.assertFalse(pydoc.ispackage(test_dir))
 
     def test_is_package_when_is_package(self):
-        with test.support.temp_cwd() as test_dir:
+        with os_helper.temp_cwd() as test_dir:
             init_path = os.path.join(test_dir, '__init__.py')
             open(init_path, 'w').close()
             self.assertTrue(pydoc.ispackage(test_dir))
@@ -1142,7 +1142,8 @@ class TestDescriptions(unittest.TestCase):
                 '''A static method'''
                 ...
         self.assertEqual(self._get_summary_lines(X.__dict__['sm']),
-                         "<staticmethod object>")
+                         'sm(x, y)\n'
+                         '    A static method\n')
         self.assertEqual(self._get_summary_lines(X.sm), """\
 sm(x, y)
     A static method
@@ -1162,7 +1163,8 @@ sm(x, y)
                 '''A class method'''
                 ...
         self.assertEqual(self._get_summary_lines(X.__dict__['cm']),
-                         "<classmethod object>")
+                         'cm(...)\n'
+                         '    A class method\n')
         self.assertEqual(self._get_summary_lines(X.cm), """\
 cm(x) method of builtins.type instance
     A class method
@@ -1569,11 +1571,20 @@ class TestInternalUtilities(unittest.TestCase):
                 self.assertIsNone(self._get_revised_path(trailing_argv0dir))
 
 
-def setUpModule():
-    thread_info = test.support.threading_setup()
-    unittest.addModuleCleanup(test.support.threading_cleanup, *thread_info)
-    unittest.addModuleCleanup(reap_children)
-
+@threading_helper.reap_threads
+def test_main():
+    try:
+        test.support.run_unittest(PydocDocTest,
+                                  PydocImportTest,
+                                  TestDescriptions,
+                                  PydocServerTest,
+                                  PydocUrlHandlerTest,
+                                  TestHelper,
+                                  PydocWithMetaClasses,
+                                  TestInternalUtilities,
+                                  )
+    finally:
+        reap_children()
 
 if __name__ == "__main__":
-    unittest.main()
+    test_main()
