@@ -1,5 +1,8 @@
 import unittest
 from test import support
+from test.support import os_helper
+from test.support import socket_helper
+from test.support import warnings_helper
 from test import test_urllib
 
 import os
@@ -47,6 +50,9 @@ class TrivialTests(unittest.TestCase):
     def test_trivial(self):
         # A couple trivial tests
 
+        # clear _opener global variable
+        self.addCleanup(urllib.request.urlcleanup)
+
         self.assertRaises(ValueError, urllib.request.urlopen, 'bogus url')
 
         # XXX Name hacking to get this to work on Windows.
@@ -158,6 +164,7 @@ class RequestHdrsTests(unittest.TestCase):
         self.assertEqual(find_user_pass("Some Realm",
                                         "http://example.com/spam"),
                          ('joe', 'password'))
+
         self.assertEqual(find_user_pass("Some Realm",
                                         "http://example.com/spam/spam"),
                          ('joe', 'password'))
@@ -166,29 +173,12 @@ class RequestHdrsTests(unittest.TestCase):
 
         add("c", "http://example.com/foo", "foo", "ni")
         add("c", "http://example.com/bar", "bar", "nini")
-        add("c", "http://example.com/foo/bar", "foobar", "nibar")
 
         self.assertEqual(find_user_pass("c", "http://example.com/foo"),
                          ('foo', 'ni'))
+
         self.assertEqual(find_user_pass("c", "http://example.com/bar"),
                          ('bar', 'nini'))
-        self.assertEqual(find_user_pass("c", "http://example.com/foo/"),
-                         ('foo', 'ni'))
-        self.assertEqual(find_user_pass("c", "http://example.com/foo/bar"),
-                         ('foo', 'ni'))
-        self.assertEqual(find_user_pass("c", "http://example.com/foo/baz"),
-                         ('foo', 'ni'))
-        self.assertEqual(find_user_pass("c", "http://example.com/foobar"),
-                         (None, None))
-
-        add("c", "http://example.com/baz/", "baz", "ninini")
-
-        self.assertEqual(find_user_pass("c", "http://example.com/baz"),
-                         (None, None))
-        self.assertEqual(find_user_pass("c", "http://example.com/baz/"),
-                         ('baz', 'ninini'))
-        self.assertEqual(find_user_pass("c", "http://example.com/baz/bar"),
-                         ('baz', 'ninini'))
 
         # For the same path, newer password should be considered.
 
@@ -647,17 +637,12 @@ class OpenerDirectorTests(unittest.TestCase):
             [("http_error_302")],
             ]
         handlers = add_ordered_mock_handlers(o, meth_spec)
-
-        class Unknown:
-            def __eq__(self, other):
-                return True
-
         req = Request("http://example.com/")
         o.open(req)
         assert len(o.calls) == 2
         calls = [(handlers[0], "http_open", (req,)),
                  (handlers[2], "http_error_302",
-                  (req, Unknown(), 302, "", {}))]
+                  (req, support.ALWAYS_EQ, 302, "", {}))]
         for expected, got in zip(calls, o.calls):
             handler, method_name, args = expected
             self.assertEqual((handler, method_name), got[:2])
@@ -782,7 +767,7 @@ class HandlerTests(unittest.TestCase):
         h = urllib.request.FileHandler()
         o = h.parent = MockOpener()
 
-        TESTFN = support.TESTFN
+        TESTFN = os_helper.TESTFN
         urlpath = sanepathname2url(os.path.abspath(TESTFN))
         towrite = b"hello, world\n"
         urls = [
@@ -1306,6 +1291,10 @@ class HandlerTests(unittest.TestCase):
 
     def test_redirect_no_path(self):
         # Issue 14132: Relative redirect strips original path
+
+        # clear _opener global variable
+        self.addCleanup(urllib.request.urlcleanup)
+
         real_class = http.client.HTTPConnection
         response1 = b"HTTP/1.1 302 Found\r\nLocation: ?query\r\n\r\n"
         http.client.HTTPConnection = test_urllib.fakehttp(response1)
@@ -1515,7 +1504,7 @@ class HandlerTests(unittest.TestCase):
             self.check_basic_auth(headers, realm)
 
         # no quote: expect a warning
-        with support.check_warnings(("Basic Auth Realm was unquoted",
+        with warnings_helper.check_warnings(("Basic Auth Realm was unquoted",
                                      UserWarning)):
             headers = [f'WWW-Authenticate: Basic realm={realm}']
             self.check_basic_auth(headers, realm)
@@ -1669,9 +1658,8 @@ class HandlerTests(unittest.TestCase):
         auth_prior_handler.add_password(
             None, request_url, user, password, is_authenticated=True)
 
-        self.assertTrue(pwd_manager.is_authenticated(request_url))
-        self.assertTrue(pwd_manager.is_authenticated(request_url + '/nested'))
-        self.assertFalse(pwd_manager.is_authenticated(request_url + 'plain'))
+        is_auth = pwd_manager.is_authenticated(request_url)
+        self.assertTrue(is_auth)
 
         opener = OpenerDirector()
         opener.add_handler(auth_prior_handler)
@@ -1802,10 +1790,8 @@ class MiscTests(unittest.TestCase):
 
     @unittest.skipUnless(support.is_resource_enabled('network'),
                          'test requires network access')
-    # bpo-46648: test fails randomly with "http://www.example.com/" URL
-    @unittest.skipIf(True, "POST request to http://www.example.com/ fail randomly")
     def test_issue16464(self):
-        with support.transient_internet("http://www.example.com/"):
+        with socket_helper.transient_internet("http://www.example.com/"):
             opener = urllib.request.build_opener()
             request = urllib.request.Request("http://www.example.com/")
             self.assertEqual(None, request.data)
